#include "gui.h"
#include "gl_window.h"
#include <GL/glut.h>
#include <FL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "partSys.h"
#include "vtkExtract.cxx"


GL_Window::GL_Window(int x, int y, int w, int h, const char *l)
: Fl_Gl_Window(x,y,w,h,l)
{
	skin_polydata = NULL;
	bone_polydata = NULL;
	GALPHA = 0.5;
	drawBone = drawSkin = 1;
	this->particles = new partSys(10,20); 
	particles->ax = particles->ay = particles->az = 0;
	particles->ttdv = 0;
	particles->dt = .01;
	particles->emmiter_pos[0] = particles->emmiter_pos[1] =
	particles->emmiter_pos[2] = 0;
	particles->scale =33;	
	
}

void GL_Window::draw() 
{
	if (!valid()) 
	{
		glClearColor(45.0f/255, 175.0f/255, 100.0f/255, 0.0);

		glEnable(GL_BLEND);
		
		glEnable(GL_DEPTH_TEST);
		glDepthFunc(GL_LESS);

		glShadeModel(GL_SMOOTH);
		glEnable(GL_LIGHTING);
		
		glViewport(0, 0, w(), h());
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(65.0, (GLfloat) w()/(GLfloat) h(), 1, 10000);
	}
	display();
}

void GL_Window::display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	//printf("calling renderer\n");
	gluLookAt(50, 0, -70, 50, 0, 0, 0, -1, 0);

	// set up lighting
	float light0ambient[] = {0.1f, 0.1f, 0.1f, 1.0};
	float light0specular[] = {2.0, 2.0, 2.0, 1.0};
 	float light0position[] = {-1.5, 1.5, -3, 0}; 
	float light0direction[] = {0.0, 0.0, -1.0};
	glLightfv(GL_LIGHT0, GL_AMBIENT, light0ambient);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light0specular);
	glLightfv(GL_LIGHT0, GL_POSITION, light0position);
	glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, light0direction);
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 180);
	glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, 2);
	glEnable(GL_LIGHT0);

	// draw test object

	glPushMatrix();
	glTranslatef(0, 0, 0);
	
	glRotated(angle, 1,0 ,0 );

	particles->update();	
	
	particles->drawIt();
	
	glTranslatef(0,-20,-20);
	if(skin_polydata != NULL){	
	drawModel(skin_polydata,0,.25,.7,GALPHA);
	}
	glPopMatrix();
	//printf("done rendering\n");
}

int GL_Window::handle(int event) 
{
	// ...
	return Fl_Gl_Window::handle(event);
}

void GL_Window::load()
{
	skin_polydata = loadHuman(*particles,isoval);
	printf("loading humand data\n");
	//bone_polydata = loadHuman(1224.5);
}

void GL_Window::drawModel(vtkPolyData* m_polydata,
float r, float g, float b,float alpha)
{
	int i, j, npts;
	float accum[3];
	GLfloat matskin[] = {r,g,b,alpha};		
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, matskin);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, matskin);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, matskin);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 128.0);

glPushMatrix();	
	// get points
	npts = m_polydata->GetNumberOfPoints();
	float *points = new float[ 3 * npts ];
	float *normals = new float[ 3 * npts ];
	for (i=0; i<npts; i++)
		m_polydata->GetPoint(i, points+3*i);

	// get triangle strips
	int nstrips = m_polydata->GetNumberOfStrips ();
	vtkCellArray *strips = m_polydata->GetStrips ();

	// get normal vectors
    vtkDataArray *normaldata = m_polydata->GetPointData()->GetNormals();
	for (i=0; i<npts; i++)
		normaldata->GetTuple (i, normals+3*i);

	// rendering
	int nvertices, *pvertices, loc = 0;
	for (i=0; i<nstrips; i++)
	{
	    strips->GetCell( loc, nvertices, pvertices ); // use GetNextCell for better performance
		//printf("nver i %d\n",nvertices);
		glBegin(GL_TRIANGLE_STRIP);
		
		for (j=0; j<nvertices; j++)
		{
			glNormal3fv(normals+3*pvertices[j]);
			glVertex3fv(points+3*pvertices[j]);
			accum[0]+=*(points+3*pvertices[j]+0);
			accum[1]+=*(points+3*pvertices[j]+1);
			accum[2]+=*(points+3*pvertices[j]+2);
			
		}
		glEnd();
		loc += nvertices + 1;
	}
	accum[0] /=(float) nvertices;
	accum[1] /=(float) nvertices;
	accum[2] /=(float) nvertices;
	printf("avg at (%f,%f,%f)\n",accum[0],accum[1],accum[2]);
	delete[] normals;
	delete[] points;
}
