#ifndef _VTK_EXTRACT_C
#define _VTK_EXTRACT_C
//#define DATA_PATH			"/tmp/womanCT/"
#define DATA_PATH "/afs/cats.ucsc.edu/users/e/wisk/public_html/womanCT_low_resolution/sub16/"

#define DATA_RESOLUTION		512		// original resolution for each slice
#define DATA_SAMPLE_RATE	16		// sub-sampling rate

#define HEADER_SIZE 0
#define RESOLUTION (DATA_RESOLUTION/DATA_SAMPLE_RATE) 
#define SPACING 1

#define FEATURE_ANGLE 90
#define DECIMATE_ANGLE FEATURE_ANGLE
#define DECIMATE_ITERATIONS 5
#define DECIMATE_ERROR 0
#define DECIMATE_ERROR_INCREMENT .0002
#define DECIMATE_REDUCTION 1
#define SMOOTH_ANGLE DECIMATE_ANGLE
#define SMOOTH_ITERATIONS 0
#define SMOOTH_FACTOR .01

#define VALUE 600.5 // 600.5 for skin, 1224.5 for bone
// refer to parts.txt for parameters of body parts
// skin0
#define NAME "skin0"
#define PIXEL_SIZE (.48828125 * DATA_SAMPLE_RATE)
#define START_SLICE 1001
#define END_SLICE 1209

#include "vtkImageReader.h"
#include "vtkMarchingCubes.h"
#include "vtkDecimate.h"
#include "vtkSmoothPolyDataFilter.h"
#include "vtkPolyDataNormals.h"
#include "vtkStripper.h"
#include "vtkPolyDataWriter.h"
#include "vtkImageData.h"
#include <time.h>
#include "partSys.h"
// utility routines and callbacks
int VOI[6] = {0, 511 / DATA_SAMPLE_RATE, 0, 425 / DATA_SAMPLE_RATE, START_SLICE, END_SLICE};
char* getTime();
void readerStart(void* data);
void mcubesStart(void* data);
void mcubesEnd(void* data);
void decimatorStart(void* data);
void decimatorEnd(void* data);
void smootherStart(void* data);
void normalsStart(void* data);
void stripperStart(void* data);

vtkPolyData* loadHuman(partSys p, float isovalue )
{
	vtkImageData *image;

	// define linear transformation
	double m[16] = {1, 0, 0, 0, 0, 0, 1, 0, 0, -1, 0, 0, 0, 0, 0, 1};
	vtkTransform *slice_order = vtkTransform::New();
	slice_order->SetMatrix(m);

	// Generate triangle data for medical images
//	float origin = ( RESOLUTION / 2.0 ) * PIXEL_SIZE * -1.0;
//	int minx = VOI[0], maxx = VOI[1], miny = VOI[2], maxy = VOI[3], minz = VOI[4], maxz = VOI[5];
	
	// adjust y bounds for lower left coordinate system
//	int tmp = miny;
//	miny = RESOLUTION - maxy - 1;
//	maxy = RESOLUTION - tmp - 1;

/*// read in image data
	vtkImageReader *reader = vtkImageReader::New();
	reader->SetDataExtent( 0, RESOLUTION - 1, 0, RESOLUTION - 1, START_SLICE, END_SLICE);
	reader->SetDataVOI( minx, maxx, miny, maxy, minz, maxz );
	reader->SetFilePrefix(DATA_PATH);
	reader->SetFilePattern ("%sc_vf%d.fre.sub16");
	reader->SetDataSpacing( PIXEL_SIZE, PIXEL_SIZE, SPACING );
	reader->SetDataOrigin( origin, origin, (START_SLICE - 1) * SPACING );
	reader->SetTransform( slice_order ); // transform the data from slice space into world space
	reader->SetHeaderSize( HEADER_SIZE ); 
	reader->SetDataMask( 0x7fff );
	reader->SetDataByteOrderToBigEndian(); // read data in UNIX byte ordering
	reader->SetDataScalarTypeToUnsignedShort(); // set the data type of pixels in the file - 16 bits
	reader->GetOutput()->GlobalReleaseDataFlagOn(); // vtkImageData::GlobalReleaseDataFlagOn(), controls whether this object's data is released after being used by a filter

	// generate isosurfaces
*/
	

	image = vtkImageData::New();
	image->SetDimensions(40,40,40);
	image->SetScalarTypeToFloat();
	image->AllocateScalars();

	float *origin = image->GetOrigin();

	printf("o at [%2f,%2f,%2f] \n",origin[0], origin[1],origin[2]);
	float *ptr = static_cast<float *>(image->GetScalarPointer());
	for(int i = 0; i < 40; i++){
	for(int j = 0; j < 40; j++){
	for(int k = 0; k < 40; k++){

		float l[3]  = {k,j-20,i-20};	// offset along -x a little bit and
	//	center on y an z
		*ptr++ = p.get_value_atv(l);
	
	}
	}
		printf("value at is %f\n",*(ptr-1));
	}
	
	vtkMarchingCubes *mcubes = vtkMarchingCubes::New();
	mcubes->SetInput(image); // vtkMarchingCubes::SetInput(vtkImageData *input)
	mcubes->ComputeScalarsOff();
	mcubes->ComputeGradientsOff();
	mcubes->ComputeNormalsOff();
	mcubes->SetValue( 0, isovalue); // Set a particular contour value at contour number i. The index i ranges between 0<=i<NumberOfContours.

	// reduce the number of triangles in a triangle mesh
	vtkDecimate *decimator = vtkDecimate::New();
	decimator->SetInput( mcubes->GetOutput() ); // vtkPolyDataToPolyDataFilter::SetInput  (  vtkPolyData *    input  ) 
	decimator->SetInitialFeatureAngle( DECIMATE_ANGLE ); // specify the mesh feature angles
	decimator->SetMaximumIterations( DECIMATE_ITERATIONS ); // Specify the maximum number of iterations to attempt. If decimation target is reached first, this value will not be reached
	decimator->SetMaximumSubIterations( 2 ); // Specify the maximum sub-iterations to perform. If no triangles are deleted in a sub-iteration, the sub-iteration process is stopped
	decimator->PreserveEdgesOn(); // Turn on the preservation of feature edges
	decimator->SetMaximumError( 1 ); // Set the largest decimation error that can be achieved by incrementing the error
	decimator->SetTargetReduction( DECIMATE_REDUCTION ); // Specify the desired reduction in the total number of polygons
	decimator->SetInitialError( DECIMATE_ERROR ); // Set the decimation error bounds
	decimator->SetErrorIncrement( DECIMATE_ERROR_INCREMENT ); // Set the value of the increment by which to increase the decimation error after each iteration
	
	// adjust point coordinates using Laplacian smoothing
	// make the cells better shaped and the vertices more evenly distributed
	vtkSmoothPolyDataFilter *smoother = vtkSmoothPolyDataFilter::New();
	smoother->SetInput( decimator->GetOutput() ); // vtkPolyDataToPolyDataFilter::SetInput  (  vtkPolyData *    input  )  
	smoother->SetNumberOfIterations( SMOOTH_ITERATIONS ); // Specify the number of iterations for Laplacian smoothing
	smoother->SetRelaxationFactor( SMOOTH_FACTOR ); // the stability of the process is sensitive to this parameter
	smoother->SetFeatureAngle( SMOOTH_ANGLE ); // Specify the feature angle for sharp edge identification
	smoother->FeatureEdgeSmoothingOff(); // Turn off smoothing along sharp interior edges
	smoother->BoundarySmoothingOff(); // Turn off the smoothing of vertices on the boundary of the mesh
	smoother->SetConvergence( 0 ); // Smaller numbers result in more smoothing iterations

	// compute point normals for a polygonal mesh
	vtkPolyDataNormals *normals = vtkPolyDataNormals::New();
	normals->SetInput( smoother->GetOutput() ); // vtkPolyDataToPolyDataFilter::SetInput  (  vtkPolyData *    input  )  
	normals->SetFeatureAngle( FEATURE_ANGLE ); // If the difference in angle across neighboring polygons is greater than this value, the shared edge is considered "sharp"

	// generate triangle strips from input
	vtkStripper *stripper = vtkStripper::New();
	stripper->SetInput( normals->GetOutput() ); // vtkPolyDataToPolyDataFilter::SetInput  (  vtkPolyData *    input  )  

	// execution time notifications
	//reader->SetStartMethod( readerStart, NULL );
	mcubes->SetStartMethod( mcubesStart, NULL );
	mcubes->SetEndMethod( mcubesEnd, mcubes );
	decimator->SetStartMethod( decimatorStart, NULL );
	decimator->SetEndMethod( decimatorEnd, decimator );
	smoother->SetStartMethod( smootherStart, NULL );
	normals->SetStartMethod( normalsStart, NULL );
	stripper->SetStartMethod( stripperStart, NULL );

	// actual execution
	stripper->Update();

	printf("done\n");
	return ( stripper->GetOutput() );
}

char* getTime()
{
	time_t ltime;
    time( &ltime );
    return ( ctime( &ltime ) );
}

void readerStart(void* data)
{
	printf( "reader start at %s\n", getTime() );
}

void mcubesStart(void* data)
{ 
	printf( "mcubes start at %s\n", getTime() );
}

void mcubesEnd(void* data)
{ 
	printf( "mcubes end with %d polygons at %s\n", ((vtkMarchingCubes*)data)->GetOutput()->GetNumberOfPolys(), getTime() );
}

void decimatorStart(void* data)
{ 
	printf( "decimator start at %s\n", getTime() );
}

void decimatorEnd(void* data)
{ 
	printf( "tdecimator end with %d polygons at %s\n", ((vtkDecimate*)data)->GetOutput()->GetNumberOfPolys(), getTime() );
}

void smootherStart(void* data)
{ 
	printf( "smoother start at %s\n", getTime() );
}

void normalsStart(void* data)
{ 
	printf( "normals start at %s\n", getTime() );
}

void stripperStart(void* data)
{ 
	printf( "stripper start at %s\n", getTime() );
}
#endif
